from collections.abc import Callable

import pytest
import yaml
from typer.testing import CliRunner, Result

from aws_helper import app, cli
from aws_helper.config import Config, Environment


@pytest.fixture(autouse=True)
def show_exceptions():
    "Configure aws_helper.cli to always show full exceptions to aid debugging."
    cli.set_show_exceptions(True)


@pytest.fixture
def cli_runner() -> CliRunner:
    return CliRunner(mix_stderr=False)


@pytest.fixture
def cli_invoke(cli_runner) -> Callable[..., Result]:
    def f(*args):
        return cli_runner.invoke(app, args)

    return f


@pytest.fixture
def isolated_filesystem(cli_runner):
    with cli_runner.isolated_filesystem():
        yield


@pytest.fixture
def write_cli_config(isolated_filesystem) -> Callable[[Config], None]:
    def f(config: Config):
        with open(".aws-helper.yaml", "w") as fobj:
            yaml.dump(config.model_dump(), fobj)

    return f


def _make_fake_environment(faker):
    return Environment(
        google_service_account_email=faker.email(),
        google_quota_project_id=faker.slug(),
        aws_role_arn=faker.bothify("arn:aws:iam::############:role/???????"),
        aws_region=faker.slug(),
    )


@pytest.fixture
def fake_cli_config(isolated_filesystem, write_cli_config, faker) -> Config:
    config = Config(
        environments={
            "development": _make_fake_environment(faker),
            faker.slug(): _make_fake_environment(faker),
        }
    )
    write_cli_config(config)
    return config
